==> tqix-3.0.1.tar.gz: tqix-3.0.1/tqix/vqa/plateau.py <==
# -*- coding: utf-8 -*-
""" usefull code for barren plateau
"""

__all__ = ['plateau']


import qiskit
import tqix.vqa.constants
import tqix.vqa.circuits
import copy

import numpy as np


def plateau(qc: qiskit.QuantumCircuit, 
            cirs,
            coefs,
            params,
            cost_func,
            num_samples):
            
    """ to run plateau
    
    Args:
        - qc (QuantumCircuit): The quantum circuit want to calculate the gradient
        - cirs (Function): circuit functions
        - coefs (Numpy array): coefs
        - params (Numpy array): Parameters
        - cost_func: name of cost function
        - num_sample = number of sample

    Returns:
        - barren plateau
    """
    
    num_qubits = qc.num_qubits
    grads = []
    
    for _ in range(num_samples):
        params = tqix.vqa.circuits.create_params(cirs,coefs,num_qubits,['random',params[1],params[2],'random'])  
        grad = gradf(qc.copy(),cirs,coefs,params,cost_func)      
        grads.append(grad) 
        
    var =  np.var(grads)       
    return var


def gradf(qc: qiskit.QuantumCircuit,cirs,coefs,params,cost_func):
    
    """Return the gradient of the loss function
        w.r.t theta_1

    Args:
        - qc (QuantumCircuit): The quantum circuit want to calculate the gradient
        - cirs (Function): circuit functions
        - coefs (Numpy array): coefs
        - params (Numpy array): Parameters
        - cost_func: name of cost function

    Returns:
        - gradient of loss function w.r.t. theta_0 
    """
    
    s = tqix.vqa.constants.step_size    
    params1, params2 = copy.deepcopy(params), copy.deepcopy(params)
    params1[0][0] += s #[0][0]the first paramater
    params2[0][0] -= s

    cost1 = cost_func(qc.copy(),cirs,coefs,params1) #ues full params
    cost2 = cost_func(qc.copy(),cirs,coefs,params2)
    
    return (cost1 - cost2)/(2*s)

==> tqix-3.0.2.tar.gz: tqix-3.0.2/tqix/vqa/plateau.py <==
# -*- coding: utf-8 -*-
""" usefull code for barren plateau
"""

__all__ = ['plateau']


import qiskit
import tqix.vqa.constants
import tqix.vqa.circuits
import copy

import numpy as np


def plateau(qc: qiskit.QuantumCircuit, 
            qcirs,
            cost_func,
            num_samples):
            
    """ to run plateau
    
    Args:
        - qc (QuantumCircuit): The quantum circuit want to calculate the gradient
        - cirs (Function): circuit functions
        - coefs (Numpy array): coefs
        - params (Numpy array): Parameters
        - cost_func: name of cost function
        - num_sample = number of sample

    Returns:
        - barren plateau
    """
    
    #num_qubits = qc.num_qubits
    grads = []
    
    for _ in range(num_samples):
        #need new params here
        param_len = len(qcirs[0][2])
        qcirs[0][2] = np.random.uniform(0, 2 * np.pi,param_len)
        qcirs[-1][2] = np.random.uniform(0, 2 * np.pi,param_len)
        
        grad = gradf(qc.copy(),qcirs,cost_func)      
        grads.append(grad) 
    var =  np.var(grads)      
    return var


def gradf(qc: qiskit.QuantumCircuit,qcirs,cost_func):
    
    """Return the gradient of the loss function
        w.r.t theta_1

    Args:
        - qc (QuantumCircuit): The quantum circuit want to calculate the gradient
        - cirs (Function): circuit functions
        - coefs (Numpy array): coefs
        - params (Numpy array): Parameters
        - cost_func: name of cost function

    Returns:
        - gradient of loss function w.r.t. theta_0 
    """
    
    s = tqix.vqa.constants.step_size    
    
    qcirs1, qcirs2 = copy.deepcopy(qcirs), copy.deepcopy(qcirs)
    qcirs1[0][2][0] += s #[0] ansatz_name, [2] ansatz_params [0] first_param
    qcirs2[0][2][0] -= s
    
    cost1 = cost_func(qc.copy(),qcirs1)
    cost2 = cost_func(qc.copy(),qcirs2)
    
    return (cost1 - cost2)/(2*s)
